using System;
using System.Text;
using System.Xml;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using System.IO;
using System.Collections.Generic;
using PluuntyContentReader.Properties;

// TODO: replace this with the type you want to read.
using TRead = Pluunty.PointerItem;

namespace PluuntyContentReader
{
    /// <summary>
    /// This class will be instantiated by the XNA Content Pipeline
    /// to read the specified data type from binary .xnb format.
    /// 
    /// Unlike the other Content Pipeline support classes, this should
    /// be a part of your main game project, and not the Content Pipeline
    /// Extension Library project.
    /// </summary>
    public class PluuntyContentReader : ContentTypeReader<TRead>
    {
        protected override TRead Read(ContentReader input, TRead existingInstance)
        {

            // calc the number of characters that will be read
            int count = (int)(input.BaseStream.Length - input.BaseStream.Position);
              
            // create a string reader to store the data that will be read
            StringBuilder sb = new StringBuilder(count, count);

            // read "count" number of characters from the input object and append the data
            // to the StringBuilder varible
            sb.Append(input.ReadChars(count));

            // create a new XmlDocument object
            XmlDocument Doc = new XmlDocument();

            // try loading the data
            try
            {
                // attempt to load the data stored in the sb object into the Doc object
                Doc.LoadXml(sb.ToString());
            }
            catch (Exception ex)
            {
                // throw a InvalidContentException exception reporting the error
                throw new XmlException(Resources.BadFileData, ex);
            }
                 
            sb = null;

            Pluunty.PointerItem pi = new Pluunty.PointerItem();
            // parse item data
            if (string.Compare(Doc.DocumentElement.Name, Resources.XML_pointer, true) == 0)
            {
                XmlNode nameNode = Doc.DocumentElement.SelectSingleNode(Resources.XML_name);
                // set name and texture if availible
                if (nameNode != null) pi.Name = nameNode.InnerText;

                XmlNode aniNode = Doc.DocumentElement.SelectSingleNode(Resources.XML_animation);
                zeeAni.AnimationSet<string, zeeAni.SpriteFrame> ani;
                string errmsg = string.Empty;
                if (aniNode != null && zeeAni.AnimationLoader<string, zeeAni.SpriteFrame>.LoadAnimation(input.ContentManager, aniNode.OuterXml, out ani, zeeAni.Converters.ConvertString, out errmsg))
                {
                    pi.Animation = ani;
                }
                else
                {
                    throw new XmlException(errmsg);
                }

                              
                
                
                /*XmlNode textureNode = Doc.DocumentElement.SelectSingleNode(Resources.XML_texture);
                XmlNode hotspotNode = Doc.DocumentElement.SelectSingleNode(Resources.XML_hotspot);

                // set name and texture if availible
                if (nameNode != null) pi.Name = nameNode.InnerText;
                if (textureNode != null) pi.TextureAssetName = textureNode.InnerText;

                Vector2 hotspot = Vector2.Zero;
                string errorMsg = string.Empty;

                // try to read hotspot
                Dictionary<string, string> att;
                if (ParseNodeAttributes(Doc.DocumentElement, Resources.XML_hotspot, out att, out errorMsg))
                {
                    if (att.ContainsKey(Resources.XML_x)) float.TryParse(att[Resources.XML_x], out hotspot.X);
                    if (att.ContainsKey(Resources.XML_y)) float.TryParse(att[Resources.XML_y], out hotspot.Y);
                }
                else
                {
                    throw new InvalidDataException(errorMsg);
                    //// try reading source rect info
                    //if (!ParseIntNode(Doc.DocumentElement, Resources.XML_x, out hs.X, out errorMsg))
                    //{
                    //    throw new InvalidDataException(errorMsg);
                    //}
                    //if (!ParseIntNode(Doc.DocumentElement, Resources.XML_y, out hs.Y, out errorMsg))
                    //{
                    //    throw new InvalidDataException(errorMsg);
                    //}
                }


                Rectangle sr = Rectangle.Empty;
                // try to read sourcerectangle element
                if (ParseNodeAttributes(Doc.DocumentElement, Resources.XML_sourcerectangle, out att, out errorMsg))
                {

                    if (att.ContainsKey(Resources.XML_x)) int.TryParse(att[Resources.XML_x], out sr.X);
                    if (att.ContainsKey(Resources.XML_y)) int.TryParse(att[Resources.XML_y], out sr.Y);
                    if (att.ContainsKey(Resources.XML_left)) int.TryParse(att[Resources.XML_left], out sr.X);
                    if (att.ContainsKey(Resources.XML_top)) int.TryParse(att[Resources.XML_top], out sr.Y);
                    if (att.ContainsKey(Resources.XML_width)) int.TryParse(att[Resources.XML_width], out sr.Width);
                    if (att.ContainsKey(Resources.XML_height)) int.TryParse(att[Resources.XML_height], out sr.Height);
                }
                else
                {
                    // try reading source rect info
                    if (!ParseIntNode(Doc.DocumentElement, Resources.XML_left, out sr.X, out errorMsg))
                    {
                        throw new InvalidDataException(errorMsg);
                    }
                    if (!ParseIntNode(Doc.DocumentElement, Resources.XML_top, out sr.Y, out errorMsg))
                    {
                        throw new InvalidDataException(errorMsg);
                    }
                    if (!ParseIntNode(Doc.DocumentElement, Resources.XML_width, out sr.Width, out errorMsg))
                    {
                        throw new InvalidDataException(errorMsg);
                    }
                    if (!ParseIntNode(Doc.DocumentElement, Resources.XML_height, out sr.Height, out errorMsg))
                    {
                        throw new InvalidDataException(errorMsg);
                    }
                }

                // set source rectange
                pi.SourceRectangle = sr;

                Color col = pi.Color;
                if (ParseColorNode(Doc.DocumentElement, Resources.XML_color, out col, out errorMsg)) throw new InvalidDataException(errorMsg);
                float tmpNum;
                if (ParseFloatNode(Doc.DocumentElement, Resources.XML_rotation, out tmpNum, out errorMsg)) pi.Rotation = tmpNum;
                if (ParseFloatNode(Doc.DocumentElement, Resources.XML_scale, out tmpNum, out errorMsg)) pi.Scale = tmpNum;
                pi.Color = col;
                pi.Hotspot = hotspot; */
            }

            // return the reference the Doc varible
            return pi;
        }

        //protected internal bool ParseFloatNode(System.Xml.XmlNode parentnode, string name, out float value)
        //{
        //    string err;
        //    return ParseFloatNode(parentnode, name, out value, out err);
        //}

        //protected internal bool ParseFloatNode(System.Xml.XmlNode parentnode, string name, out float value, out string err)
        //{
        //    value = new float();
        //    System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
        //    if (node == null)
        //    {
        //        err = Resources.CouldNotFindNamedElement.Replace(Resources.RKEY, name);
        //        return false;
        //    }
        //    if (!float.TryParse(node.InnerText, out value))
        //    {
        //        err = Resources.ValueIsNotFloat + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
        //        return false;
        //    }
        //    err = string.Empty;
        //    return true;
        //}

        //protected internal bool ParseColorNode(System.Xml.XmlNode parentnode, string name, out Color value, out string err)
        //{
        //    Dictionary<string, string> items;
        //    if (!ParseNodeAttributes(parentnode, name, out items, out err))
        //    {
        //        value = Color.White;
        //        err = Resources.ExpectedNumber + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
        //        return false;
        //    }

        //    byte rv, gv, bv, av;
        //    rv = 255;
        //    gv = 255;
        //    bv = 255;
        //    av = 255; // default alpha value

        //    // ensure that aleast the rgb components were specified
        //    if (!items.ContainsKey(Resources.XML_r) || !items.ContainsKey(Resources.XML_g) || !items.ContainsKey(Resources.XML_b))
        //    {
        //        value = Color.White;
        //        err = Resources.ExpectedNumber + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
        //        return false;
        //    }

        //    // get rbg values
        //    if (!byte.TryParse(items[Resources.XML_r], out rv))
        //    {
        //        value = Color.White;
        //        err = Resources.ExpectedNumber + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
        //        return false;
        //    }
        //    if (!byte.TryParse(items[Resources.XML_g], out gv))
        //    {
        //        value = Color.White;
        //        err = Resources.ExpectedNumber + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
        //        return false;
        //    }
        //    if (!byte.TryParse(items[Resources.XML_b], out bv))
        //    {
        //        value = Color.White;
        //        err = Resources.ExpectedNumber + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
        //        return false;
        //    }

        //    // get alpha value
        //    if (items.ContainsKey(Resources.XML_a) && !byte.TryParse(items[Resources.XML_a], out av))
        //    {
        //        value = Color.White;
        //        err = Resources.ExpectedNumber + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
        //        return false;
        //    }

        //    // create color
        //    value = new Color(rv, gv, bv, av);

        //    return true;
        //}

        //protected internal bool ParseNodeAttributes(System.Xml.XmlNode parentnode, string name, out Dictionary<string, string> value)
        //{
        //    string err;
        //    return ParseNodeAttributes(parentnode, name, out value, out err);
        //}

        //protected internal bool ParseNodeAttributes(System.Xml.XmlNode parentnode, string name, out Dictionary<string, string> value, out string err)
        //{
        //    value = null;
        //    System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
        //    if (node == null)
        //    {
        //        err = Resources.CouldNotFindNamedElement.Replace(Resources.RKEY, name);
        //        return false;
        //    }
        //    ParseAttributes(node, out value);
        //    err = string.Empty;
        //    return true;
        //}

        //protected internal void ParseAttributes(System.Xml.XmlNode node, out Dictionary<string, string> list)
        //{

        //    Dictionary<string, string> items = new Dictionary<string, string>();
        //    for (int idx = 0; idx <= node.Attributes.Count - 1; idx++)
        //    {
        //        items.Add(node.Attributes[idx].Name, node.Attributes[idx].InnerText);
        //    }
        //    list = items;
        //}

        //protected internal bool ParseIntNode(System.Xml.XmlNode parentnode, string name, out int value)
        //{
        //    string err;
        //    return ParseIntNode(parentnode, name, out value, out err);
        //}

        //protected internal bool ParseIntNode(System.Xml.XmlNode parentnode, string name, out int value, out string err)
        //{
        //    value = new int();
        //    System.Xml.XmlNode node = parentnode.SelectSingleNode(name);
        //    if (node == null)
        //    {
        //        err = Resources.CouldNotFindNamedElement.Replace(Resources.RKEY, name);
        //        return false;
        //    }
        //    if (!int.TryParse(node.InnerText, out value))
        //    {
        //        err = Resources.ValueIsNotInteger + " " + Resources.NamedElement.Replace(Resources.RKEY, name);
        //        return false;
        //    }
        //    err = string.Empty;
        //    return true;
        //}
    }
}
